@@ -63,10 +63,16 @@
 #include <stdio.h>
 #include <errno.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 char *program_name;
 const char *kernel_dir;
 const char *image_file;
 int flags = GST_NO_TTY;
+int run_as_daemon;
+int usage;
 
 struct tool {
   const char *name;
@@ -86,13 +92,15 @@ struct tool tools[] = {
   {
     "gst-load", "scripts/Load.st",
     "-h|--help --version -q|--quiet -v|-V|--verbose -n|--dry-run -f|--force \
-	-t|--test -I|--image-file: --kernel-directory: -i|--rebuild-image",
+	--start:: -t|--test -I|--image-file: --kernel-directory: \
+	-i|--rebuild-image",
     NULL
   },
   {
     "gst-reload", "scripts/Load.st",
     "-h|--help --version -q|--quiet -v|-V|--verbose -n|--dry-run -f|--force \
-	-t|--test -I|--image-file: --kernel-directory: -i|--rebuild-image",
+	--start:: -t|--test -I|--image-file: --kernel-directory: \
+	-i|--rebuild-image",
     "--force"
   },
   {
@@ -120,6 +128,13 @@ struct tool tools[] = {
         -n|--namespace: -o|--output: --kernel-directory: -F|--output-format:",
     NULL
   },
+  {
+    "gst-remote", "scripts/Remote.st",
+    "-h|--help --version --daemon --server -p|--port: -f|--file: -e|--eval: \
+ 	-l|--login: --package: --start: --stop: --pid --kill --snapshot:: \
+	-I|--image-file: --kernel-directory:",
+    NULL
+  },
   { NULL, NULL, NULL, NULL }
 };
 
@@ -127,10 +142,10 @@ struct tool tools[] = {
    Does not support canonical option names, otherwise it is pretty
    complete.  */
  
-struct option {
+struct long_option {
   char arg;
   const char *name;
-  struct option *next;
+  struct long_option *next;
 };
 
 #define OPT_NONE	0
@@ -138,7 +153,7 @@ struct option {
 #define OPT_OPTIONAL	2
 
 char short_opts[1 << (sizeof (char) * 8)];
-struct option *long_opts;
+struct long_option *long_opts;
 
 void
 option_error (const char *s, ...)
@@ -191,7 +206,7 @@ setup_option (char *p, char *end)
 
       if (long_opt)
 	{
-	  struct option *opt = malloc (sizeof (struct option));
+	  struct long_option *opt = malloc (sizeof (struct long_option));
 	  opt->name = strdup (long_opt);
 	  opt->arg = arg;
 	  opt->next = long_opts;
@@ -247,8 +262,62 @@ parse_option (int short_opt, const char *long_opt, const char *arg)
   if (short_opt == 'i'
       || (long_opt && !strcmp (long_opt, "rebuild-image")))
     flags |= GST_REBUILD_IMAGE;
+
+  if (long_opt && !strcmp (long_opt, "daemon"))
+    {
+#ifdef HAVE_FORK
+      run_as_daemon = 1;
+#else
+      fprintf (stderr, "Daemon operation not supported.");
+      exit (77);
+#endif
+    }
+
+  if (long_opt && !strcmp (long_opt, "version"))
+    usage = 1;
+
+  if (short_opt == 'h'
+      || (long_opt && !strcmp (long_opt, "help")))
+    usage = 1;
 }
 
+#ifdef HAVE_FORK
+static void
+fork_daemon (void)
+{
+  int child_pid;
+
+#ifdef SIGHUP
+  signal (SIGHUP, SIG_IGN);
+#endif
+
+  child_pid = fork();
+  if (child_pid < 0)
+    {
+      perror("Failed to fork daemon");
+      exit(1);
+    }
+
+  /* Stop parent.  */
+  if (child_pid != 0)
+    exit (0);
+
+  /* Detach and spawn server.
+     Create a new SID for the child process */
+#ifdef HAVE_SETSID
+  if (setsid() < 0)
+    {
+      perror("setsid failed");
+      exit(1);
+    }
+#endif
+
+#ifdef SIGHUP
+  signal (SIGHUP, SIG_DFL);
+#endif
+}
+#endif
+
 int
 parse_short_options (const char *name, const char *arg)
 {
@@ -278,7 +347,7 @@ parse_short_options (const char *name, const char *arg)
 int
 parse_long_option (const char *name, const char *arg)
 {
-  struct option *all_opts, *opt = NULL;
+  struct long_option *all_opts, *opt = NULL;
   int num_matches = 0;
   int len;
   const char *p = strchr (name, '=');
@@ -408,6 +477,11 @@ main(int argc, const char **argv)
   setup_options (tools[i].options);
   parse_options (&argv[1]);
 
+#ifdef HAVE_FORK
+  if (run_as_daemon && !usage)
+    fork_daemon ();
+#endif
+
   if (tools[i].force_opt)
     {
       smalltalk_argv = alloca (sizeof (const char *) * (argc + 1));
@@ -433,6 +507,7 @@ main(int argc, const char **argv)
 
   gst_set_var (GST_VERBOSITY, 0);
   gst_smalltalk_args (smalltalk_argc, smalltalk_argv);
+  gst_set_executable_path (argv[0]);
   result = gst_initialize (kernel_dir, image_file, flags);
   if (result != 0)
     exit (result < 0 ? 1 : result);
