@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2003, 2006, 2007 Free Software Foundation, Inc.
+ * Copyright 2003, 2006, 2007, 2008 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -55,7 +55,7 @@
 
 #include "config.h"
 
-#ifdef HAVE_INET_SOCKETS
+#ifdef HAVE_SOCKETS
 
 #ifdef __MSVCRT__
 #include <windows.h>
@@ -71,7 +71,7 @@
 #define ECONNABORTED WSAECONNABORTED
 #define ENETRESET WSAENETRESET
 
-typedef int nfds_t;
+typedef int socklen_t;
 
 /* re-define FD_ISSET to avoid a WSA call while we are not using 
  * network sockets */
@@ -92,14 +92,13 @@ win_fd_isset (int fd, fd_set * set)
   return 0;
 }
 
-int win_select(int n, fd_set *rfds, fd_set *wfds, fd_set *efds, struct timeval *ptv);
-int win_recv(int fd, void* buffer, int n, int flags);
+#define FD_TO_SOCKET(fd)   ((SOCKET) _get_osfhandle ((fd)))
+#define SOCKET_TO_FD(fh)   (_open_osfhandle ((HANDLE) (fh), O_RDWR | O_BINARY))
 
-#undef select
-#define select win_select
+#undef close
+#define close		   win_close
 
-#undef recv
-#define recv win_recv
+extern int win_close(int filehandle);
 
 #else /* !__MSVCRT__ */
 
@@ -111,15 +110,23 @@ int win_recv(int fd, void* buffer, int n, int flags);
 #include <sys/select.h>
 #endif
 #include <sys/socket.h>
-#include <netdb.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 
-#define closesocket(x)            close(x)
 #define is_socket_error(err)      (errno == (err))
 #define clear_socket_error()      (errno = 0)
+
+#define FD_TO_SOCKET(fd)	  (fd)
+#define SOCKET_TO_FD(fd)	  (fd)
+typedef int SOCKET;
+
+#ifndef SOCKET_ERROR
+#define SOCKET_ERROR		  (-1)
+#endif
 #endif /* !__MSVCRT__ */
 
-#endif /* HAVE_INET_SOCKETS */
+#include "getaddrinfo.h"
+
+#endif /* HAVE_SOCKETS */
 
 #endif /* GST_SOCKETX_H */
