@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2008
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -103,24 +103,23 @@ cdata_type;
 extern int _gst_errno 
   ATTRIBUTE_HIDDEN;
 
-/* Element type for the name-to-C-function mapping table.  */
-typedef void (*p_void_func) ();
-
-typedef struct gst_cfunc_descriptor
+typedef struct gst_c_callable
 {
   OBJ_HEADER;
-  OOP cFunction;                /* gst_cobject whose C value is func
-                                   addr */ 
-  OOP cFunctionName;            /* Name of C function in mapping table */
-  OOP returnType;               /* Smalltalk return type */
-  OOP numFixedArgs;             /* number of real arguments passed from
-                                   smalltalk (excluding "self" parameters
-                                   which are synthetically added when 
-                                   calling the C function).  */
-  OOP argTypes[1];              /* variable length, really numFixedArgs
-                                   long */
+  OOP typeOOP;		        /* CObject fields */ 
+  OOP storageOOP;	        /* CObject fields */ 
+  OOP returnTypeOOP;            /* Smalltalk return type */
+  OOP argTypesOOP;		/* array of argument types */
+  OOP blockOOP;			/* only for CCallbackDescriptor */
 }
- *gst_cfunc_descriptor;
+ *gst_c_callable;
+
+/* Returns the size of an object passed to a C routine with type TYPE.  */
+extern int _gst_c_type_size (int type);
+
+/* Called after GC to invalidate the cache for the libffi representation
+   of CFunctionDescriptors.  */
+extern void _gst_invalidate_croutine_cache (void);
 
 /* Invokes a C routine.  Arguments passed from Smalltalk are stored starting
    from ARGS, and the object to which the message that called-out was
@@ -146,19 +145,23 @@ extern void _gst_define_cfunc (const char *funcName, PTR funcAddr)
 extern void _gst_init_cfuncs (void) 
   ATTRIBUTE_HIDDEN;
 
-/* Makes a C based descriptor for a callout method.  Returns a
-   gst_cfunc_descriptor object which holds onto the descriptor.  This
-   descriptor is subsequently used when the called out function
-   FUNCNAMEOOP (a Smalltalk String) is invoked. RETURNOOP is a Symbol
-   or CType which indicates the return type and ARGSOOP is a Smalltalk
-   Array containing the argument types (as Symbols).  */
-extern OOP _gst_make_descriptor (OOP funcNameOOP,
-				 OOP returnTypeOOP,
-				 OOP argsOOP) 
-  ATTRIBUTE_HIDDEN;
-
 /* Set the value of errno which is checked by Smalltalk to be errnum.  */
 extern void _gst_set_errno(int errnum) 
   ATTRIBUTE_HIDDEN;
 
+/* Returns the address for the latest C function which has been
+   registered using _gst_define_cfunc with the name FUNCNAME.  Returns
+   NULL if there is no such function.  */
+extern PTR _gst_lookup_function (const char *funcName)
+  ATTRIBUTE_HIDDEN;
+
+/* Creates a closure for the CCallbackDescriptor CALLBACKOOP and stores it
+   in the object.  */
+extern void _gst_make_closure (OOP callbackOOP)
+  ATTRIBUTE_HIDDEN;
+
+/* Frees the info for the closure in the CCallbackDescriptor CALLBACKOOP.  */
+extern void _gst_free_closure (OOP callbackOOP)
+  ATTRIBUTE_HIDDEN;
+
 #endif /* GST_CINT_H */
