@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2000, 2001, 2002, 2003, 2005, 2006 Free Software Foundation, Inc.
+ * Copyright 2000, 2001, 2002, 2003, 2005, 2006, 2008 Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
  * This file is part of GNU Smalltalk.
@@ -69,14 +69,15 @@
 typedef struct gst_file_stream
 {
   OBJ_HEADER;
+  OOP access;
+  OOP fd;
+  OOP file;
+  OOP isPipe;
+  OOP atEnd;
+  OOP peek;
   OOP collection;
   OOP ptr;
   OOP endPtr;
-  OOP access;
-  OOP file;
-  OOP name;
-  OOP isPipe;
-  OOP peek;
   OOP writePtr;
   OOP writeEnd;
 }
@@ -111,6 +112,7 @@ typedef struct gst_namespace
   OOP superspace;
   OOP name;
   OOP subspaces;
+  OOP sharedPools;
   OOP assoc[1];
 
   /* Other, indexable fields that are the associations for this
@@ -130,9 +132,9 @@ typedef struct gst_identity_dictionary
 #define BEHAVIOR_HEADER \
   OBJ_HEADER; \
   OOP		superclass; \
-  OOP		subClasses; \
   OOP		methodDictionary; \
   intptr_t	instanceSpec; \
+  OOP		subClasses; \
   OOP		instanceVariables
 
 typedef struct gst_behavior
@@ -158,6 +160,34 @@ typedef struct gst_behavior
 #define COBJECT_INT_TYPE		FROM_INT(10)
 #define COBJECT_UNSIGNED_INT_TYPE	FROM_INT(11)
 
+/*    3                   2                   1
+    1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   |    # fixed fields             |      unused       |I| kind  |1|
+   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+   I'm moving it to bits 13-30 (it used to stay at bits 5-30),
+   allocating space for more flags in case they're needed.
+   If you change ISP_NUMFIXEDFIELDS you should modify Behavior.st too.
+   Remember to shift by ISP_NUMFIXEDFIELDS-1 there, since Smalltalk does
+   not see GST_ISP_INTMARK!!  */
+
+enum {
+  ISP_NUMFIXEDFIELDS = 13,
+
+  /* Set if the instances of the class have indexed instance
+     variables.  */
+  ISP_ISINDEXABLE = 32,
+
+  /* These represent the shape of the indexed instance variables of
+     the instances of the class.  */
+  ISP_INDEXEDVARS = 62,
+  ISP_SHAPE = 30,
+
+  /* Set to 1 to mark a SmallInteger.  */
+  GST_ISP_INTMARK = 1
+};
+
 typedef struct gst_class_description
 {
   CLASS_DESCRIPTION_HEADER;
@@ -291,6 +321,7 @@ typedef struct gst_cobject
 {
   OBJ_HEADER;
   OOP type;
+  OOP storage;
 }
  *gst_cobject;
 
@@ -314,8 +345,9 @@ extern OOP _gst_block_closure_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_block_context_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_boolean_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_byte_array_class ATTRIBUTE_HIDDEN;
-extern OOP _gst_byte_stream_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_c_callable_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_c_func_descriptor_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_c_callback_descriptor_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_c_object_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_c_object_type_ctype ATTRIBUTE_HIDDEN;
 extern OOP _gst_c_type_class ATTRIBUTE_HIDDEN;
@@ -349,6 +381,7 @@ extern OOP _gst_homed_association_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_identity_dictionary_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_identity_set_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_integer_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_iterable_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_interval_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_large_integer_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_large_negative_integer_class ATTRIBUTE_HIDDEN;
@@ -515,12 +548,14 @@ extern OOP _gst_shared_pool_dictionary (OOP class_oop)
   ATTRIBUTE_PURE 
   ATTRIBUTE_HIDDEN;
 
-/* Creates a new CObject pointing to cObjPtr, extracting the class
+/* Creates a new CObject pointing to cObjOfs bytes in BASEOOP (or
+   at the absolute address cObjOfs if BASEOOP is NULL), extracting the class
    to be instantiated from the CType, TYPEOOP, or using the provided
    class if TYPEOOP is nil.  */
-extern OOP _gst_c_object_new (PTR cObjPtr,
-			      OOP typeOOP,
-			      OOP defaultClassOOP) 
+extern OOP _gst_c_object_new_base (OOP baseOOP,
+			           uintptr_t cObjOfs,
+			           OOP typeOOP,
+			           OOP defaultClassOOP) 
   ATTRIBUTE_HIDDEN;
 
 /* Creates a new String with LEN indexed instance variables.  */
