@@ -90,9 +90,6 @@ const char *_gst_image_file_path = NULL;
    home directory.  */
 const char *_gst_user_file_base_path = NULL;
 
-/* The path to the executable, derived from argv[0].  */
-const char *_gst_executable_path = NULL;
-
 /* Whether to look for user files.  */
 static mst_Boolean no_user_files = false;
 
@@ -191,14 +188,15 @@ static const char standard_files[] = {
   "Link.st\0"
   "Process.st\0"
   "CallinProcess.st\0"
-  "CompildCode.st\0"
-  "CompildMeth.st\0"
-  "CompiledBlk.st\0"
+  "Iterable.st\0"
   "Collection.st\0"
   "SeqCollect.st\0"
   "LinkedList.st\0"
   "Semaphore.st\0"
   "ArrayColl.st\0"
+  "CompildCode.st\0"
+  "CompildMeth.st\0"
+  "CompiledBlk.st\0"
   "Array.st\0"
   "ByteArray.st\0"
   "CharArray.st\0"
@@ -226,7 +224,6 @@ static const char standard_files[] = {
   "ReadStream.st\0"
   "WriteStream.st\0"
   "RWStream.st\0"
-  "ByteStream.st\0"
   "UndefObject.st\0"
   "ProcSched.st\0"
   "Delay.st\0"
@@ -264,14 +261,18 @@ static const char standard_files[] = {
   /* C call-out facilities */
   "CObject.st\0"
   "CType.st\0"
+  "CCallable.st\0"
   "CFuncs.st\0"
+  "CCallback.st\0"
   "CStruct.st\0"
 
-  /* Exception handling */
+  /* Exception handling and ProcessEnvironment */
+  "ProcEnv.st\0"
   "ExcHandling.st\0"
   "AnsiExcept.st\0"
 
   /* Virtual filesystem layer */
+  "FilePath.st\0"
   "File.st\0"
   "Directory.st\0"
   "VFS.st\0"
@@ -308,7 +309,6 @@ _gst_smalltalk_args (int argc,
 {
   smalltalk_argc = argc;
   smalltalk_argv = argv;
-  _gst_executable_path = _gst_find_executable (argv[0]);
 }
 
    
@@ -330,6 +330,9 @@ _gst_initialize (const char *kernel_dir,
   _gst_smalltalk_initialized = true;
   _gst_init_snprintfv ();
 
+  if (!_gst_executable_path)
+    _gst_executable_path = DEFAULT_EXECUTABLE;
+
   /* By default, apply this kludge fpr OSes such as Windows and MS-DOS
      which have no concept of home directories.  */
   if (home == NULL)
@@ -352,6 +355,8 @@ _gst_initialize (const char *kernel_dir,
 	}
     }
 
+  /* For the image file, it is okay to find none if we can/should rebuild
+     the image file.  */
   if (image_file
       && (flags & (GST_REBUILD_IMAGE | GST_MAYBE_REBUILD_IMAGE)) == 0
       && !_gst_file_is_readable (image_file))
@@ -365,6 +370,8 @@ _gst_initialize (const char *kernel_dir,
 	}
     }
 
+  /* The image path can be used as the default kernel path, so we split
+     it anyway into directory+filename.  */
   if (image_file)
     {
       const char *p;
@@ -395,26 +402,31 @@ _gst_initialize (const char *kernel_dir,
     }
   else
     {
-      /* No image file given, we use the system default but revert to the
+      /* No image file given, we use the system default or revert to the
 	 current directory.  */
-      if (_gst_file_is_readable (IMAGE_PATH))
-        _gst_image_file_path = IMAGE_PATH;
+      str = _gst_relocate_path (IMAGE_PATH);
+      if (_gst_file_is_readable (str))
+        _gst_image_file_path = str;
       else
-        _gst_image_file_path = xstrdup (currentDirectory);
+	{
+          free (str);
+          _gst_image_file_path = xstrdup (currentDirectory);
+	}
+
       flags |= GST_IGNORE_BAD_IMAGE_PATH;
       image_file = "gst.im";
     }
 
   if (!kernel_dir)
     {
-      if (_gst_file_is_readable (KERNEL_PATH))
-        kernel_dir = KERNEL_PATH;
-      else
+      str = _gst_relocate_path (KERNEL_PATH);
+      if (!_gst_file_is_readable (str))
 	{
-          char *kernel_file_path;
-	  asprintf (&kernel_file_path, "%s/kernel", _gst_image_file_path);
-	  kernel_dir = kernel_file_path;
+          free (str);
+	  asprintf (&str, "%s/kernel", _gst_image_file_path);
 	}
+
+      kernel_dir = str;
     }
 
   xfree (currentDirectory);
@@ -570,6 +582,8 @@ load_standard_files (void)
 	}
     }
 
+  _gst_msg_sendf (NULL, "%v %o relocate", _gst_file_segment_class);
+
   if (site_pre_image_file)
     _gst_process_file (site_pre_image_file, GST_DIR_ABS);
 
