@@ -11,7 +11,7 @@
 
 /***********************************************************************
  *
- * Copyright 2007 Free Software Foundation, Inc.
+ * Copyright 2007, 2008 Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
  * This file is part of GNU Smalltalk.
@@ -110,7 +110,15 @@ VMProxy gst_interpreter_proxy = {
   _gst_process_stdin,
   _gst_process_file,
   _gst_get_var, _gst_set_var,
-  _gst_invoke_hook
+  _gst_invoke_hook,
+
+  /* New in 3.1.  */
+  _gst_relocate_path,
+  _gst_oop_indexed_base,
+  _gst_oop_indexed_kind,
+  _gst_async_call,
+  _gst_sync_signal,
+  _gst_show_backtrace
 };
 
 /* Functions in comp.h.  */
@@ -380,6 +388,24 @@ gst_async_signal_and_unregister (OOP semaphore_oop)
   _gst_async_signal_and_unregister (semaphore_oop);
 }
 
+mst_Boolean
+gst_sync_signal (OOP semaphore_oop, mst_Boolean incr_if_empty)
+{
+  return _gst_sync_signal (semaphore_oop, incr_if_empty);
+}
+
+void
+gst_async_call (void (*func) (OOP), OOP semaphore_oop)
+{
+  _gst_async_call (func, semaphore_oop);
+}
+
+void
+gst_show_backtrace (FILE *fp)
+{
+  _gst_show_backtrace (fp);
+}
+
 void
 gst_register_oop_array (OOP **first, OOP **last)
 {
@@ -499,6 +525,31 @@ gst_oop_at_put (OOP oop, size_t index, OOP new_oop)
   return _gst_oop_at_put (oop, index, new_oop);
 }
 
+enum gst_indexed_kind
+gst_oop_indexed_kind (OOP oop)
+{
+  return _gst_oop_indexed_kind (oop);
+}
+
+void *
+gst_oop_indexed_base (OOP oop)
+{
+  return _gst_oop_indexed_base (oop);
+}
+
+
+/* Functions in sysdep.h.  */
+void
+gst_set_executable_path (const char *argv0)
+{
+  _gst_set_executable_path (argv0);
+}
+
+char *
+gst_relocate_path (const char *path)
+{
+  return _gst_relocate_path (path);
+}
 
 static void
 init_vmproxy (void)
