@@ -8,7 +8,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2008
  * Free Software Foundation, Inc.
  * Written by Steve Byrne and Paolo Bonzini.
  *
@@ -63,11 +63,13 @@
 
 #ifdef __cplusplus
 #include <cstddef>
+#include <cstdio>
 extern "C"
 {
 #else
 
 #include <stddef.h>
+#include <stdio.h>
 #endif
 
 #include "gst.h"
@@ -203,6 +205,9 @@ typedef struct VMProxy
   char *(*relocatePath) (const char *);
   void *(*OOPIndexedBase) (OOP oop);
   enum gst_indexed_kind (*OOPIndexedKind) (OOP oop); 
+  void (*asyncCall) (void (*func) (OOP), OOP argOOP);
+  mst_Boolean (*syncSignal) (OOP semaphoreOOP, mst_Boolean incrIfEmpty);
+  void (*showBacktrace) (FILE *fp);
 } VMProxy;
 
 /* Compatibility section */
@@ -277,7 +282,10 @@ extern char *gst_oop_to_string (OOP oop);
 extern char *gst_oop_to_byte_array (OOP oop);
 extern PTR gst_oop_to_c_object (OOP oop);
 extern void gst_async_signal (OOP semaphore_oop);
+extern void gst_async_call (void (*func) (OOP), OOP arg_oop);
+extern mst_Boolean gst_sync_signal (OOP semaphore_oop, mst_Boolean incr_if_empty);
 extern void gst_sync_wait (OOP semaphore_oop);
+extern void gst_show_backtrace (FILE *fp);
 extern void gst_async_signal_and_unregister (OOP semaphore_oop);
 extern void gst_register_oop_array (OOP **first, OOP **last);
 extern void gst_unregister_oop_array (OOP **first);
