@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007,2008
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -328,8 +328,10 @@ extern const char *_gst_abort_execution
 
 /* Set to true when some special action must be done at the next
    sequence point.  */
-extern volatile mst_Boolean _gst_except_flag 
+#ifdef ENABLE_JIT_TRANSLATION
+extern mst_Boolean _gst_except_flag 
   ATTRIBUTE_HIDDEN;
+#endif
 
 /* Create a new Process on the top of the stack, which is specially
    marked so that it stops the interpreter's execution.  This kind
@@ -384,6 +386,12 @@ extern void _gst_init_interpreter (void)
 extern void _gst_empty_context_pool (void) 
   ATTRIBUTE_HIDDEN;
 
+/* Set up so that FUNC will be called, with ARGOOP as its argument,
+   as soon as the next sequence point is reached.  */
+extern void _gst_async_call (void (*func) (OOP),
+			     OOP argOOP) 
+  ATTRIBUTE_HIDDEN;
+
 /* Signal SEMAPHOREOOP so that one of the processes waiting on that
    semaphore is waken up.  Since a Smalltalk call-in is not an atomic
    operation, the correct way to signal a semaphore is not to send
@@ -409,7 +417,7 @@ extern void _gst_invalidate_method_cache (void)
 
 /* Show a backtrace of the current state of the stack of execution
    contexts.  */
-extern void _gst_show_backtrace (void) 
+extern void _gst_show_backtrace (FILE *) 
   ATTRIBUTE_HIDDEN;
 
 /* Trap the signals that we care about, basically SIGBUS and
@@ -513,6 +521,18 @@ extern void _gst_send_method (OOP methodOOP)
 extern void _gst_sync_wait (OOP semaphoreOOP) 
   ATTRIBUTE_HIDDEN;
 
+/* Signal the given SEMAPHOREOOP and if processes were queued on it
+   resume the one that has waited for the longest time and is still
+   alive.  If INCR is true, increment its value if no processes were
+   queued.  Return true if a process was woken.
+
+   This functions also cannot be called from within a signal handler.
+   It can be called from a function that is registered with
+   _gst_async_call, though.  */
+extern mst_Boolean _gst_sync_signal (OOP semaphoreOOP,
+			      mst_Boolean incr_if_empty)
+  ATTRIBUTE_HIDDEN;
+
 /* Take a CompiledBlock and turn it into a BlockClosure that references
    thisContext as its static link.  */
 extern OOP _gst_make_block_closure (OOP blockOOP) 
