@@ -148,18 +148,17 @@
 static int tm_diff (struct tm *a,
 		    struct tm *b);
 
-static void do_interrupts (mst_Boolean disable);
-
 #define DISABLED_MASK ((-1) ^ (1 << SIGSEGV) ^ (1 << SIGBUS) ^ \
-			      (1 << SIGILL) ^ (1 << SIGABRT))
+			      (1 << SIGQUIT) ^ (1 << SIGILL) ^ (1 << SIGABRT))
 
 
 #if defined SIG_BLOCK
-#define DECLARE \
-  sigset_t newSet; \
-  static sigset_t oldSet
-  
+static sigset_t oldSet;
+
+#define DECLARE
+
 #define DISABLE do { \
+  sigset_t newSet; \
   sigfillset (&newSet); \
   sigdelset (&newSet, SIGSEGV); \
   sigdelset (&newSet, SIGBUS); \
@@ -173,9 +172,11 @@ static void do_interrupts (mst_Boolean disable);
   sigprocmask (SIG_SETMASK, &oldSet, NULL)
 
 #elif defined HAVE_SIGSETMASK     /* BSD */
-#define DECLARE static int mask
-#define DISABLE mask = sigsetmask (DISABLED_MASK)
-#define ENABLE sigsetmask (mask)
+static int signalMask;
+
+#define DECLARE
+#define DISABLE signalMask = sigsetmask (DISABLED_MASK)
+#define ENABLE sigsetmask (signalMask)
 
 #elif defined HAVE_SIGHOLD 	/* SVID style */
 #define DECLARE int i
@@ -228,31 +229,40 @@ static RETSIGTYPE dummy_signal_handler (int sig)
   while (0)
 #endif
 
+int _gst_signal_count;
+
 void
-do_interrupts (mst_Boolean disable)
+_gst_disable_interrupts (mst_Boolean from_signal_handler)
 {
   DECLARE;
-  static int count = 0;
   
-  /* let only the outermost calls actually block and unblock
-     signals */
-  if (disable && count++ == 0)
-    DISABLE;
-    
-  else if (!disable && --count == 0)
-    ENABLE;
+  __sync_synchronize ();
+  if (_gst_signal_count++ == 0)
+    {
+      __sync_synchronize ();
+#ifdef _POSIX_VERSION
+      if (from_signal_handler)
+        return;
+#endif
+      DISABLE;
+    }
 }
 
 void
-_gst_disable_interrupts ()
+_gst_enable_interrupts (mst_Boolean from_signal_handler)
 {
-  do_interrupts (true);
-}
+  DECLARE;
 
-void
-_gst_enable_interrupts ()
-{
-  do_interrupts (false);
+  __sync_synchronize ();
+  if (--_gst_signal_count == 0)
+    {
+      __sync_synchronize ();
+#ifdef _POSIX_VERSION
+      if (from_signal_handler)
+        return;
+#endif
+      ENABLE;
+    }
 }
 
 #undef DECLARE
@@ -271,7 +281,12 @@ _gst_set_signal_handler (int signum,
   act.sa_handler = handlerFunc;
   act.sa_flags = 0;
 
-  sigemptyset (&act.sa_mask);
+  sigfillset (&act.sa_mask);
+  sigdelset (&act.sa_mask, SIGSEGV);
+  sigdelset (&act.sa_mask, SIGBUS);
+  sigdelset (&act.sa_mask, SIGILL);
+  sigdelset (&act.sa_mask, SIGQUIT);
+  sigdelset (&act.sa_mask, SIGABRT);
   sigaction (signum, &act, &o_act);
   return o_act.sa_handler;
 
@@ -1010,6 +1025,9 @@ _gst_file_is_executable (const char *fileName)
 static int executable_fd = -1;
 #endif
 
+/* The path to the executable, derived from argv[0].  */
+const char *_gst_executable_path = NULL;
+
 /* Tests whether a given pathname may belong to the executable.  */
 static mst_Boolean
 maybe_executable (const char *filename)
@@ -1043,8 +1061,8 @@ maybe_executable (const char *filename)
    Return NULL if unknown.  Guaranteed to work on Linux and Win32, Mac OS X.
    Likely to work on the other Unixes (maybe except BeOS), under most
    conditions.  */
-char *
-_gst_find_executable (const char *argv0)
+static char *
+find_executable (const char *argv0)
 {
 #if defined WIN32
   char location[MAX_PATH];
@@ -1103,7 +1121,7 @@ _gst_find_executable (const char *argv0)
     n = readlink (buf, location, path_max);
     if (n > 0 && location[0] != '[')
       {
-        location[n] == '\0';
+        location[n] = '\0';
         return location;
       }
     if (executable_fd < 0)
@@ -1160,6 +1178,46 @@ _gst_find_executable (const char *argv0)
 #endif
 }
 
+void
+_gst_set_executable_path (const char *argv0)
+{
+  _gst_executable_path = find_executable (argv0);
+}
+
+char *
+_gst_relocate_path (const char *path)
+{
+  const char *p;
+  char *s;
+
+  /* Detect absolute paths.  */
+#if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
+  if ((path[0] && path[1] == ':')
+      || path[0] == '/' || path[0] == '\\')
+    return xstrdup (path);
+#else
+  if (path[0] == '/')
+    return xstrdup (path);
+#endif
+
+  /* Remove filename from executable path.  */
+  p = _gst_executable_path + strlen (_gst_executable_path);
+  do
+    --p;
+  while (p >= _gst_executable_path
+         && *p != '/'
+#if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
+	 && *p != '\\'
+#endif
+	);
+  p++;
+
+  /* Now p points just past the last separator (if any).  */
+  s = alloca (p - _gst_executable_path + strlen (path) + 1);
+  sprintf (s, "%.*s%s", p - _gst_executable_path, _gst_executable_path, path);
+  return _gst_get_full_file_name (s);
+}
+
 
 /* Code to use PTY's did not work on Mac OS.  I'm keeping the Unix code,
    but it should not be used.  */
@@ -1470,6 +1528,7 @@ _gst_open_pipe (const char *command,
     return -1;
 
   _gst_set_signal_handler (SIGPIPE, SIG_DFL);
+  _gst_set_signal_handler (SIGFPE, SIG_DFL);
 
 #ifdef HAVE_SPAWNL
   init_pty (&pty);
@@ -1513,6 +1572,7 @@ _gst_open_pipe (const char *command,
 #endif /* !HAVE_SPAWNL */
 
   _gst_set_signal_handler (SIGPIPE, SIG_IGN);
+  _gst_set_signal_handler (SIGFPE, SIG_IGN);
 
   /* Free it in the parent too! */
   free (pty.slavenam);
@@ -1537,7 +1597,6 @@ _gst_open_pipe (const char *command,
 {
   int fd[2];
   int our_fd, child_fd;
-  int master;
   int access;
   int result;
 
@@ -1561,6 +1620,7 @@ _gst_open_pipe (const char *command,
     return -1;
 
   _gst_set_signal_handler (SIGPIPE, SIG_DFL);
+  _gst_set_signal_handler (SIGFPE, SIG_DFL);
 
 #ifdef HAVE_SPAWNL
   {
@@ -1627,6 +1687,7 @@ _gst_open_pipe (const char *command,
 
   close (child_fd);
   _gst_set_signal_handler (SIGPIPE, SIG_IGN);
+  _gst_set_signal_handler (SIGFPE, SIG_IGN);
 
   if (result == -1)
     {
@@ -1693,6 +1754,8 @@ _gst_open_file (const char *filename,
 
 #if defined FD_CLOEXEC && !defined O_CLOEXEC
   fcntl (fd, F_SETFD, fcntl (fd, F_GETFD, 0) | FD_CLOEXEC);
+#elif defined __MSVCRT__
+  SetHandleInformation (_get_osfhandle (fd), HANDLE_FLAG_INHERIT, 0);
 #endif
 
   return fd;
@@ -1717,14 +1780,14 @@ _gst_recv (int fd,
 	   int size,
 	   int flags)
 {
-#ifdef HAVE_INET_SOCKETS
+#ifdef HAVE_SOCKETS
   int result;
   int save_errno = errno;
 
   for (;;)
     {
-      result = recvfrom (fd, buffer, size, flags, NULL, NULL);
-      if (errno == EFAULT)
+      result = recvfrom (FD_TO_SOCKET (fd), buffer, size, flags, NULL, NULL);
+      if (is_socket_error (EFAULT))
         abort ();
 
       if (is_socket_error (EINTR))
@@ -1749,14 +1812,14 @@ _gst_send (int fd,
 	   int size,
 	   int flags)
 {
-#ifdef HAVE_INET_SOCKETS
+#ifdef HAVE_SOCKETS
   int result;
   int save_errno = errno;
 
   for (;;)
     {
-      result = send (fd, buffer, size, flags);
-      if (errno == EFAULT)
+      result = send (FD_TO_SOCKET (fd), buffer, size, flags);
+      if (is_socket_error (EFAULT))
         abort ();
 
       if (is_socket_error (EINTR))
@@ -1861,6 +1924,7 @@ _gst_init_sysdep (void)
   tzset ();
 
   _gst_set_signal_handler (SIGPIPE, SIG_IGN);
+  _gst_set_signal_handler (SIGFPE, SIG_IGN);
   _gst_set_signal_handler (SIGCHLD, sigchld_handler);
 }
 
