@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007,2008
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -81,17 +81,26 @@
 
 typedef RETSIGTYPE (*SigHandler) ();
 
+/* The path to the executable.  */
+extern const char *_gst_executable_path;
+
+/* The count of nested _gst_disable_interrupts calls.  */
+extern int _gst_signal_count;
+
 /* Saves and returns the current state of the software interrupt
    system.  Disables all interrupts.  */
-extern void _gst_disable_interrupts (void)
+extern void _gst_disable_interrupts (mst_Boolean fromSignalHandler)
   ATTRIBUTE_HIDDEN;
 
 /* Restores the state of the interrupt system to that which it had when
    _gst_disable_interrupts was last invoked.  */
-extern void _gst_enable_interrupts (void)
+extern void _gst_enable_interrupts (mst_Boolean fromSignalHandler)
   ATTRIBUTE_HIDDEN;
 
-/* Set the signal handler for signal SIGNUM to be HANDLERFUNC.  */
+/* Set the signal handler for signal SIGNUM to be HANDLERFUNC.  Further
+   interrupts are blocked within the signal if supported.  (The signal
+   handler should still call _gst_disable_interrupts and
+   _gst_enable_interrupts passing true to fromSignalHandler).  */
 extern SigHandler _gst_set_signal_handler (int signum,
 					   SigHandler handlerFunc)
   ATTRIBUTE_HIDDEN;
@@ -185,7 +194,12 @@ extern mst_Boolean _gst_file_is_executable (const char *fileName)
   ATTRIBUTE_HIDDEN;
 
 /* Return a path to the executable given argv[0].  */
-extern char *_gst_find_executable (const char *argv0)
+extern void _gst_set_executable_path (const char *argv0)
+  ATTRIBUTE_HIDDEN;
+
+/* Return the absolute path for PATH, interpreted relative to the
+   executable.  */
+char *_gst_relocate_path (const char *path)
   ATTRIBUTE_HIDDEN;
 
 /* Answer true if the file descriptor FD is associated to a pipe
